/*
 * Copyright (c) 2011 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-tlscnf-0.c,v 1.5 2012/11/05 15:24:58 ca Exp $")

#include "sm/error.h"
#include "sm/io.h"
#include "sm/sm-conf.h"
#include "sm/sm-conf-prt.h"
#include "sm/smtpdef.h"
#define SM_CONF_TIME_DEF 1
#include "sm/sm-conf-time.h"
#define SM_TLSCNFDEF	1
#include "sm/tlscnf.h"
#include "sm/tlscnfs.h"
#include "sm/misc.h"
#include "sm/test.h"

#if MTA_USE_TLS
static int Verbose;

typedef struct ss_cnf_S		ss_cnf_T, *ss_cnf_P;
struct ss_cnf_S
{
	tls_cnf_T	 ss_cnf_tls;
};

sm_conf_definition_T
t_tls_defs[] = {

{ SM_CONF_DEF_MAGIC, "tls", sm_conf_type_section,
	offsetof(ss_cnf_T, ss_cnf_tls), sizeof(ss_cnf_T), NULL,
	SM_CONF_FLAG_KEEP_DEFAULT,
	tls_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
CONF_SENTINEL
};

static sm_ret_T
test(char *data, bool isfile)
{
	int err;
	ss_cnf_T ss_cnf;
	FILE *fp;
	sm_conf_T *smc;

	fp = NULL;
	sm_memzero(&ss_cnf, sizeof(ss_cnf));
	smc = sm_conf_new(data);
	if (smc == NULL) {
		err = errno;

		sm_io_fprintf(smioerr,
			"sev=ERROR, func=sm_readconf, file=%s, sm_conf_new=NULL, errno=%d\n",
			data, err);
		return ENOMEM;
	}
	if (isfile)
		err = sm_conf_read_FILE(smc, data, fp);
	else
		err = sm_conf_read_data(smc, data, strlen(data), false);
	if (err != 0) {
		sm_prt_conferr(data, smc, err, smioerr);
		sm_conf_destroy(smc);
		smc = NULL;
		return err;
	}

	err = sm_conf_scan(smc, t_tls_defs, 0, &ss_cnf);
	if (err != 0)
		sm_prt_conferr(data, smc, err, smioerr);
	else {
		sm_str_P text;
		dhparam_T	 dhparam;

		text = sm_str_new(NULL, 256, 8192);
		(void) sm_conf_prt_cnfs(t_tls_defs, &ss_cnf, 0, smioout, 0, NULL, text, 0);
		sm_io_flush(smioout);

		dhparam = ss_cnf.ss_cnf_tls.tlscnf_dhparam;
		switch (dhparam.dhparam_type) {
		  case DHPBITS:
			sm_io_fprintf(smioerr, "dhparam: bits=%d\n",
				dhparam.dhbits.dhbits_bits);
			break;
		  case DHPFILE:
			sm_io_fprintf(smioerr, "dhparam: file=%s\n",
				dhparam.dhpfile.dhpfile_name);
			break;
		  default:
			sm_io_fprintf(smioerr, "dhparam: not-set\n");
			break;
		}
	}
	return 0;
}

int
main(int argc, char *argv[])
{
	int c, r;
	bool isfile;
	char *prg;

	opterr = 0;
	Verbose = 0;
	isfile = true;
	prg = argv[0];
	while ((c = getopt(argc, argv, "dVv")) != -1)
	{
		switch (c)
		{
		  case 'd':
			isfile = false;
			break;
		  case 'v':
			compoptions(smioerr);
			break;
		  case 'V':
			++Verbose;
			break;
		  default:
			exit(1);
		}
	}
	sm_test_begin(argc, argv, "test tls conf 0");
	argc -= optind;
	argv += optind;
	if (argc > 0) {
		for (r = 0; r < argc; r++) {
			test(argv[r], isfile);
		}
	}
	return sm_test_end();
}
#else /* MTA_USE_TLS */
int
main(int argc, char *argv[])
{
	sm_io_fprintf(smioerr, "MTA_USE_TLS not set\n");
	return 0;
}
#endif /* MTA_USE_TLS */
