#!/bin/sh
# $Id: t-priset-0.sh,v 1.2 2011/05/11 23:13:42 ca Exp $
#
# Copyright (c) 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test overriding priority by cltaddr
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto smtps COMMERCIAL
chkcto smtps SMI_PRI

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# clean up before running tests
cd ../qmgr || exit 1

# create directory to pass socket
test -d smtps || mkdir smtps

killit()
{
rm -f ${QMGRCONFMAP} ${MTMAP}
exit 1
}

${CAT} > ${QMGRCONF} <<EOF
qmgr_conf_cltaddr:127.0.0.1	pri=10;
EOF
# create map
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}
rm -f ${MTMAP}
trap killit 2

# ----------------------------------------
# XREF: local.dom: mt
${CAT} >${SMXCNF} <<EOF
smtps { log_level = 14; wait_for_server=4;
	relay_to = ".*@local\.dom.*";
	daemon_address = localhost:${SRVPORT};
	path = ../smtps/smtps;
	arguments = "smtps -f ${SMXCNF} ${MTA_SERVER_OPTIONS}";
}
smtpc { log_level = 14; remote_port = ${SNKPORT}; wait_for_server=4;
	connect_only_to = 127.0.0.1;
	threads_max = 1; # low number to delay delivery
	io_timeout=60s;
	path = "../smtpc/smtpc"; arguments = "smtpc -f ${SMXCNF}";
}
qmgr { log_level = 14; wait_for_server=4; wait_for_client=4; 
	flags = lookup_qmgr_conf;
	path = "../qmgr/qmgr"; arguments = "qmgr -f ${SMXCNF}";
}
smar { log_level = 14;
	path = "../smar/smar"; arguments = "smar -f ${SMXCNF}";
}
EOF

# XREF: local.dom: relaying
${CAT} > ${MT} <<EOF
b.c	[127.0.0.1]
local.dom	[127.0.0.1]
EOF

# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}

clearlogs

# ----------------------------------------
TEST=1
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
ERR=false
# 
if ${SHELL} ${SD}/smtpcnf.sh -Z smtpc2 -1 -Sa10@b.c -Rpri10@local.dom -E 1 -X 1 -A 1
then
  if grep ', pri=10, ' ${QML} >/dev/null
  then
    :
  else
    echo "$0: missing pri=10 in ${QML}"
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
if ${ERR}
then
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
else
  ${VERBOSE} && echo "$0: test ${TEST} succeeded"
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
TEST=2
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
ERR=false
getip2 2

${CAT} > ${QMGRCONF} <<EOF
qmgr_conf_cltaddr:127.0.0.1	pri=10;
qmgr_conf_cltaddr:$H2	pri=2;
EOF
# create map
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}

if ${SHELL} ${SD}/smtpcnf.sh -Z smtpc2 -1 -Sa2@b.c -Rpri2@local.dom -E 1 -X 1 -A 1 -c "-b $H2 -d 4"
then
  if grep ', pri=2, ' ${QML} >/dev/null
  then
    :
  else
    echo "$0: missing pri=2 in ${QML}"
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
if ${ERR}
then
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
else
  ${VERBOSE} && echo "$0: test ${TEST} succeeded"
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
