/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: backslash.c,v 1.2 2010/06/10 03:44:09 ca Exp $")

#include "sm/limits.h"
#include "sm/string.h"
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/memops.h"
#include "sm/ctype.h"
#include <stdio.h>
#include "sm/sm-conf.h"

#include "sm-conf-token.h"

/*
**  SM_CONF_BACKSLASH -- convert a backslash token in a string
**
**	Parameter:
**		s_ptr -- where to write
**		r_ptr -- where to read
**		endptr -- end of input
**
**	Returns:
**		0 on success.
**		SM_CONF_ERR_* on error
*/

SM_CONF_ERROR_T
sm_conf_backslash(char **s_ptr, char const **r_ptr, char const *endptr)
{
	unsigned long	ul = 0;
	size_t			i;
	SM_CONF_ERROR_T	err;
	char			*s = *s_ptr;
	char const		*r = *r_ptr;

	err = 0;
	ul = 0;
	i = 4;

	if (r >= endptr)
		err = SM_CONF_ERR_EOF_IN_STRING;
	else if (IS_NEWLINE(r, endptr))
		err = SM_CONF_ERR_NEWLINE_IN_STRING;
	else {
		switch (*r++) {
		  case 'a':
			*s++ = '\a';
			break;
		  case 'b':
			*s++ = '\b';
			break;
		  case 'f':
			*s++ = '\f';
			break;
		  case 'n':
			*s++ = '\n';
			break;
		  case 'r':
			*s++ = '\r';
			break;
		  case 't':
			*s++ = '\t';
			break;
		  case 'v':
			*s++ = '\v';
			break;

		  case 'x':
			/* \xNNNNNN... */
			if (r >= endptr) {
				*s_ptr = s;
				*r_ptr = r;
				return SM_CONF_ERR_EOF_IN_STRING;
			}
			if (!ISXDIGIT(*r)) {
				r++;
				err = SM_CONF_ERR_HEX_EXPECTED;
				break;
			}

			while (r < endptr && ISXDIGIT(*r)) {
				ul = (ul << 4) | sm_conf_token_hexvalue(*r++);
				if ((ul >> CHAR_BIT) != 0) {
					err = SM_CONF_ERR_CHAR_OVERFLOW;
					break;
				}
			}
			if (err == 0)
				*s++ = (unsigned char)ul;
			break;

		  case 'U':
			i = 8;
		  case 'u':
			/* \u1234, \U12345678 */

			while (i-- > 0) {
				if (r > endptr) {
					r = endptr;
					err = SM_CONF_ERR_EOF_IN_STRING;
					break;
				}
				if (!ISXDIGIT(*r)) {
					err = SM_CONF_ERR_HEX_EXPECTED;
					r++;
					break;
				}
				ul = (ul << 4) | sm_conf_token_hexvalue(*r++);
			}

			if (err == 0)
				/* UTF-8 encode <ul> */
				err = sm_conf_token_ulong_to_utf8(ul, &s);
			break;

		  default:
			r--;
			if (*r < '0' || *r > '7') {
				*s++ = '\\';
				*s++ = *r++;
			}
			else {
				/* \oct [oct [oct]] */

				ul = *r++ - '0';
				if (r < endptr && *r >= '0' && *r <= '7') {
					ul = (ul << 3) | (*r++ - '0');
					if (r < endptr && *r >= '0' && *r <= '7')
						ul = (ul << 3) | (*r++ - '0');
				}
				if ((ul >> CHAR_BIT) > 0) {
					err = SM_CONF_ERR_CHAR_OVERFLOW;
					break;
				}
				*s++ = ul;
			}
			break;
		}
	}

	if (err == 0)
		*s_ptr = s;
	else {
		if (r > *r_ptr)
			/* copy the spelling of the problematic token */
			sm_memcpy(*s_ptr, *r_ptr, r - *r_ptr);
		*s_ptr += r - *r_ptr;
	}
	*r_ptr = r;
	return err;
}
