/*
 * Copyright (c) 2011 Claus Assmann
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the license/LICENSE.3C file which can be found at the
 * top level of this source code distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-inh-0.c,v 1.5 2012/11/05 15:24:57 ca Exp $")

#if SM_LIBCONF_ALONE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sm-conf.h"
#else /* SM_LIBCONF_ALONE */
#include "sm/io.h"
#include "sm/string.h"
#include "sm/net.h"
#include "sm/units.h"
#include "sm/sm-conf.h"
#define SM_CONF_BYTE_DEF 1
#define SM_CONF_KBYTE_DEF 1
#include "sm/sm-conf-byte.h"
#include "sm/sm-conf-prt.h"
#include "sm/io.h"
#endif /* SM_LIBCONF_ALONE */

/*
**  testing partial inheritance
*/

static int Verbose = 0;

#define SSC_CFL_DELAY_CHKS	0x00000002u
#define SSC_CFL_ACCESS		0x00000004u
#define SSC_CFL_STARTTLS	0x00800000u
#define SSC_CFL_AUTH		0x01000000u

#define SM_SRV_FEAT_MAGIC SM_MAGIC('S', 'R', 'V', 'F')
#define SM_CNF_MAGIC SM_MAGIC('S', 'C', 'N', 'F')

#if SM_LIBCONF_MAGIC
#define CNF_MAGIC , SM_CNF_MAGIC
#else
#define CNF_MAGIC 
#endif

struct tls_cnf_S
{
	char		*tlscnf_cert;
	char		*tlscnf_key;
	char		*tlscnf_cacert;
	uint32_t	 tlscnf_bits;
};

typedef struct tls_cnf_S tls_cnf_T;

sm_conf_definition_T
tls_defs[]
=
{
{ SM_CONF_DEF_MAGIC, "cert_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_cert),	0,
	"cert.pem",
	SM_CONF_FLAG_CHK0,	NULL,	NULL,	NULL,
	"File with certificate in PEM format"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
,{ SM_CONF_DEF_MAGIC, "key_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_key),	0,
	"key.pem",
	SM_CONF_FLAG_CHK0,	NULL,	NULL,	NULL,
	"File with private key for certificate in PEM format"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
,{ SM_CONF_DEF_MAGIC, "CAcert_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_cacert),	0,
	"ca.pem",
	0,	NULL,	NULL,	NULL,
	"File with CA certificate in PEM format"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
,{ SM_CONF_DEF_MAGIC, "bits",		sm_conf_type_u32,
	offsetof(tls_cnf_T, tlscnf_bits),	sizeof(uint32_t),
	NULL,	0,	NULL, NULL,	NULL,
	"bits"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }
,CONF_SENTINEL
}
;

/* session features */
struct ss_srv_feat_S {
	char		*sssf_name;
	uint32_t	 sssf_flags;
	uint32_t	 sssf_max_ta;
	tls_cnf_T	 sssf_tls;
#if SM_LIBCONF_ISSET
	uchar		 sssf_isset_name;
	uchar		 sssf_isset_flags;
	uchar		 sssf_max_ta_isset;
#endif
};
typedef struct ss_srv_feat_S ss_srv_feat_T, *ss_srv_feat_P;

/* array of session features */
struct ss_srv_feats_S {
	ss_srv_feat_T  *sssfs_feat;
	uint            sssfs_n;
};
typedef struct ss_srv_feats_S ss_srv_feats_T, *ss_srv_feats_P;

typedef struct ss_cnf_S		ss_cnf_T, *ss_cnf_P;
struct ss_cnf_S
{
	sm_magic_T	 sm_magic;
	uint32_t	 ss_cnf_cflags;
	uint32_t	 ss_cnf_max_ta;
	tls_cnf_T	 ss_cnf_tls;
	ss_srv_feats_T	 ss_cnf_srv_feats;
#if SM_LIBCONF_ISSET
	uchar		 sssf_isset_cflags;
	uchar		 ss_cnf_max_ta_isset;
#endif
};

sm_conf_definition_T const ssc_flag_names[] = {
{ SM_CONF_DEF_MAGIC, "u|",  sm_conf_type_u32_suffix, 1,
	1, NULL, 0, NULL, NULL, NULL, "" },
{ SM_CONF_DEF_MAGIC, "|",  sm_conf_type_u32_suffix, 1,
	1, NULL, 0, NULL, NULL, NULL, "" },
{ SM_CONF_DEF_MAGIC, "u",  sm_conf_type_u32_suffix, 1,
	1, NULL, 0, NULL, NULL, NULL, "" },
{ SM_CONF_DEF_MAGIC,
	"access",		sm_conf_type_choice_value,
	SSC_CFL_ACCESS,
	0, NULL, 0, NULL, NULL, NULL,
	"use access map" },
{ SM_CONF_DEF_MAGIC,
	"starttls",		sm_conf_type_choice_value,
	SSC_CFL_STARTTLS,
	0, "starttls", SM_CONF_FLAG_CHK0, NULL, NULL, NULL,
	"offer STARTTLS" },
{ SM_CONF_DEF_MAGIC,
	"auth",		sm_conf_type_choice_value,
	SSC_CFL_AUTH,
	0, "auth", SM_CONF_FLAG_CHK0, NULL, NULL, NULL,
	"offer AUTH" },
{ SM_CONF_DEF_MAGIC,
	"delay_checks",		sm_conf_type_choice_value,
	SSC_CFL_DELAY_CHKS,
	0, NULL, SM_CONF_FLAG_CHK0, NULL, NULL, NULL,
	"delay returning results from access check until RCPT stage" },
CONF_SENTINEL
};

sm_conf_definition_T ss_srv_feat_defs[] = {
{ SM_CONF_DEF_MAGIC, "name",		sm_conf_type_section_title,
	offsetof(ss_srv_feat_T, sssf_name),	0,
	NULL,	0 /*SM_CONF_FLAG_STRICTLY_REQUIRED*/,
	NULL, NULL, NULL, NULL
	SM_LC_ISSET(offsetof(ss_srv_feat_T, sssf_isset_name))
	},
{ SM_CONF_DEF_MAGIC, "flags",		sm_conf_type_choice,
	offsetof(ss_srv_feat_T, sssf_flags),	sizeof(uint32_t),
	/* SM_XSTR(SSC_CFL_STARTTLS) "|" SM_XSTR(SSC_CFL_AUTH) */
	/* "0x00800000|0x01000000" "{starttls,auth}"*/
	NULL,
	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR|SM_CONF_FLAG_PLURAL|
		SM_CONF_FLAG_NUMERIC|SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT|SM_CONF_FLAG_SET0,
	ssc_flag_names,
	NULL,	NULL,
	"flags to influence behavior per session"
	SM_LC_ISSET(offsetof(ss_srv_feat_T, sssf_isset_flags))
	},
{ SM_CONF_DEF_MAGIC, "max_ta",		sm_conf_type_u32,
	offsetof(ss_srv_feat_T, sssf_max_ta),	sizeof(uint32_t),
	NULL, SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT,
	NULL, NULL,	NULL,
	"max ta"
	SM_LC_ISSET(offsetof(ss_srv_feat_T, sssf_max_ta_isset))
	},
{ SM_CONF_DEF_MAGIC, "tls", sm_conf_type_section,
	offsetof(ss_srv_feat_T, sssf_tls), sizeof(ss_srv_feat_T), NULL,
	SM_CONF_FLAG_KEEP_DEFAULT|SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_SET0,
	tls_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};

sm_conf_definition_T ss_srv_feat_definitions[] = {
{ SM_CONF_DEF_MAGIC, "",	sm_conf_type_section,
	offsetof(ss_cnf_T, ss_cnf_srv_feats.sssfs_feat), sizeof(ss_srv_feat_T),
	NULL, 0,
	ss_srv_feat_defs,
	NULL, NULL, "session features" },
{ SM_CONF_DEF_MAGIC, "n",	sm_conf_type_array_n,
	offsetof(ss_cnf_T, ss_cnf_srv_feats.sssfs_n), sizeof(uint),
	NULL, 0,
	ss_srv_feat_defs,
	NULL, NULL,
	"number of session features (implicit)" },
CONF_SENTINEL
};

sm_conf_definition_T ss_defs[] = {
{ SM_CONF_DEF_MAGIC, "session_features",		sm_conf_type_array,
	0,	/* offset */
	0,	/* size (# of elements) */
	NULL,	/* default */
	SM_CONF_FLAG_MULTIPLE,
	ss_srv_feat_definitions,
	NULL,  NULL,
	"session feature declarations"	},
{ SM_CONF_DEF_MAGIC, "flags",		sm_conf_type_choice,
	offsetof(ss_cnf_T, ss_cnf_cflags),	sizeof(uint32_t),
	SM_XSTR(SSC_CFL_STARTTLS) /* "|" SM_XSTR(SSC_CFL_AUTH) */,
	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR|SM_CONF_FLAG_PLURAL|SM_CONF_FLAG_NUMERIC,
	ssc_flag_names,
	NULL,	NULL,
	"global flags to influence behavior"
	SM_LC_ISSET(offsetof(ss_cnf_T, sssf_isset_cflags))
	CNF_MAGIC
	},
{ SM_CONF_DEF_MAGIC, "max_ta",		sm_conf_type_u32,
	offsetof(ss_cnf_T, ss_cnf_max_ta),	sizeof(uint32_t),
	"99",	0,
	NULL, NULL,	NULL,
	"global max ta"
	SM_LC_ISSET(offsetof(ss_cnf_T, ss_cnf_max_ta_isset))
	CNF_MAGIC
	},

{ SM_CONF_DEF_MAGIC, "tls", sm_conf_type_section,
	offsetof(ss_cnf_T, ss_cnf_tls), sizeof(ss_cnf_T), NULL,
	SM_CONF_FLAG_KEEP_DEFAULT|SM_CONF_FLAG_MULTIPLE,
	tls_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(SM_SC_CNF_MAGIC) },

CONF_SENTINEL
};

static ss_cnf_T s;

static int
process(char const *name, FILE *fp)
{
	sm_conf_T		*smc;
	uint n, u;
	int err;
	char buf[SM_CONF_ERROR_BUFFER_SIZE];
	char const *e = NULL;

	s.sm_magic = SM_CNF_MAGIC;
	if (((smc = sm_conf_new(name ? name : "*stdin*"))) == NULL) {
		sm_io_fprintf(smioerr, "error -- sm_conf_new() returns NULL!\n");
		return 1;
	}
	if ((err = sm_conf_read_FILE(smc, name, fp)) != 0) {
		sm_io_fprintf(smioerr, "%s: %s\n", name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));
		while ((e = sm_conf_syntax_error(smc, e)) != NULL)
			sm_io_fprintf(smioerr, "%s\n", e);
		sm_conf_destroy(smc);
		return 2;
	}

	if ((err = sm_conf_scan(smc, ss_defs, 0, &s)) != 0) {
		sm_io_fprintf(smioerr, "(while scanning) %s: %s\n", name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));
		while ((e = sm_conf_syntax_error(smc, e)) != NULL)
			sm_io_fprintf(smioerr, "%s\n", e);
		sm_conf_destroy(smc);
		return 3;
	}

	(void) sm_conf_prt_conf(ss_defs, &s, smioerr);
	if (Verbose > 1) {
		n = s.ss_cnf_srv_feats.sssfs_n;
		sm_io_fprintf(smioerr, "size=%lu\n", sizeof(ss_srv_feat_T));
		for (u = 0; u < n; u++) {
			sm_io_fprintf(smioerr, "addr[%u]=%p\n", u,
				&(s.ss_cnf_srv_feats.sssfs_feat[u]));
			sm_io_fprintf(smioerr, "name[%u]=%s\n", u,
				s.ss_cnf_srv_feats.sssfs_feat[u].sssf_name);
			sm_io_fprintf(smioerr, "flags[%u]=0x%08x (%p)\n", u,
				s.ss_cnf_srv_feats.sssfs_feat[u].sssf_flags,
				&(s.ss_cnf_srv_feats.sssfs_feat[u].sssf_flags));
		}
	}

#if 0
	sm_conf_destroy(smc);
#endif

	return 0;
}

int
main(int ac, char **av)
{
	int ai, c, ret, done;
	char *ssname;

	done = 0;
	ssname = NULL;
	while ((c = getopt(ac, av, "f:N:")) != -1) {
		switch (c) {
		  case 'f':
			ret = process(optarg, NULL);
			if (ret != 0)
				return ret;
			done = 1;
			break;
		  case 'N':
			ssname = strdup(optarg);
			if (ssname == NULL)
				return ENOMEM;
			break;
		}
	}
	ac -= optind;
	av += optind;

	if (ac == 0)
		return process("*stdin*", stdin);

	for (ai = 0; ai < ac; ai++) {
		int ret = process(av[ai], NULL);
		if (ret != 0)
			return ret;
	}
	return 0;
}
