#!/bin/sh
# $Id: t-dkim-1.sh,v 1.14 2011/01/14 17:55:41 ca Exp $
# Copyright (c) 2010 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test dkim signing, "simple" body/header canonicalization
# dealing with empty lines at end of body.
# ----------------------------------------

test -s stop && exit 1              

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../libdkim
fi
ERRS=0
PRG=./t-dkim-1
BODY=msg
OUT=t-dkim-1.out

if $PRG -C | grep MTA_USE_DKIM >/dev/null
then
  :
else
   echo "SKIPPED: $0: $PRG not compiled with MTA_USE_DKIM"
   exit 0
fi

VERBOSE=false
if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

emptylines()
{
 ELINES=$1
el=0
while test $el -lt $ELINES
do
cat >> $BODY <<EOF

EOF
el=`expr $el + 1 `
done

cat >> $BODY <<EOF
.
EOF
}

# ----------------------------------------
TEST=1
ERR=false
EL=0

H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="hU1eiDdhQCwLZh1jXoUNF9Sg++8hJfXb4823xLTxNK2y6nMc6M6/iRudoNbF8/ijn/by/PnVE94Jphmzbr9OCfwh0zy+L9wd8ajH7ge7zPjQgUvuiLIzf33GtHYSrSGG4ry+LkUJNcKvjxI7UDSRlnodX6X3V6dWpto6l06yNeU="

while test $EL -le 5
do
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

cat >$BODY <<EOF
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
EOF

emptylines $EL

$PRG -k $SD/private.key -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
  break
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

EL=`expr $EL + 1 `
TEST=`expr $TEST + 1 `

done

# ----------------------------------------
ERR=false
EL=0

H="/e7NVSO8ZkgZhPLKmpfOxkTADL6kfTUa93kUY6vA720="
SIG="kAPrqxEChrgYL73kMKYIel5DLCTCb9St0P7F9KRMMr8YqqY9xOVLWDHvmUyoPFvqF92caE4KdG7KX4T/iZOfPbHcV59+JwBn03Lbd5zzrd4KzK4XZQXK5lq920hCBczHvu5t5SWD7k+x3kUPg0klpdu7Jf/TAHTgqeuw9BktYys="

while test $EL -le 5
do
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
cat >$BODY <<EOF
From: "This A. User" <user@example.com>
To: Not Really <recipient@sendmail.net>
Subject: test DKIM signing with MeTA1
Message-Id: <12345.abcde@example.net>
Date: Fri, 17 Dec 2010 17:20:05 -0800 (PST)

This is the message body. It's nice, isn't it?
It does not contain anything useful, but that is the case with most e-mails.

-- 
2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
EOF

emptylines $EL

$PRG -k $SD/private.key -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
  break
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

EL=`expr $EL + 1 `
TEST=`expr $TEST + 1 `

done

# ----------------------------------------
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="LaQ8THTVGANkTj2kCIz75kFryBJ9T0xfUu249GpX+bJNbBEz55fug7DC73aBJQWk6DRv4Yz61CgRShUKx0igNaEIY2G9dzNcPO2zLKo2OsW20U28mrnRtTd6KS2IPjWhZ/pMhxeBaAJe8li8U3m0/moUJrPMTqjzJs4MZiUHYfY="

cat >$BODY <<EOF
From: user@example.com
To: recipient@sendmail.net,
	another@example.net
Subject: test
	DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

if $PRG -k $SD/private.key -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi
TEST=`expr $TEST + 1 `

# ----------------------------------------
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

H="0UJfgPAKs0CsoCKDyE2F/DURzgkyqelLb9RVT2C4wDQ="
SIG="ZPh316pbuKjME69mcQ69/L649ykQTpB+D2p6Gy5MiAsFk/dH7E/TeKLgv9al/+gcvbx6G3OgUP9f1vY9onfWJWHLbB47W5OVu1NFVMyXuWAmLgOyr0udkYvJlFUeFXaq6Npyfe8iajIjnfozIAAuqU6lRwFaOtpGw4s26L+AKwY="

cat >$BODY <<EOF
From: "This A. User" <user@example.net>
To: Not Really <recipient@sendmail.net>, But A Lst <r2@example.com>
Subject: Testing DKIM signing with MeTA1
Message-Id: <123456789.abcdeXYZ@example.net>
Date: Fri, 27 Dec 2010 17:20:05 -0000 (GMT)

This is the message body. It's nice, isn't it?
It does not contain anything useful,
but that is the case with most e-mails.

More message content. To fill up the buffer.
More message content. To fill up the buffers.
More message content. To fill up the buffer.
More message content. To fill up the buffers.
More message content. To fill up the buffer.
More message content. To fill up the buffers.
More message content. To fill up the buffer.
More message content. To fill up the buffers.
More message content. To fill up the buffer.
More message content. To fill up the buffers.
More message content. To fill up the buffer.
More message content. To fill up the buffers.
More message content. To fill up the buffer.
More message content. To fill up the buffers.
More message content. To fill up the buffer.
More message content. To fill up the buffers.
More message content. To fill up the buffer.
More message content. To fill up the buffers.
More message content. To fill up the buffer.
More message content. To fill up the buffers.
-- 
This might be a signature virus
.
EOF

if $PRG -k $SD/net.key -s net -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi
TEST=`expr $TEST + 1 `


# ----------------------------------------
if test ${ERRS} -eq 0
then
  rm -f ${BODY} ${OUT}
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
