/*
 * Copyright (c) 2011 Claus Assmann
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the license/LICENSE.3C file which can be found at the
 * top level of this source code distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-edbc-perf-0.c,v 1.4 2013/10/23 22:13:46 ca Exp $")

#include <stdio.h>

#include "sm/assert.h"
#include "sm/string.h"
#include "sm/sysexits.h"
#include "sm/test.h"
#include "sm/edbc.h"
#include "edbc.h"

#define KEYS_MAX	256
static time_T next_try[KEYS_MAX];
uint qm_debug = 0;

static void
test(uint maxn, uint nodes, bool check)
{
	int a;
	uint u;
	sm_ret_T ret;
	sessta_id_T ta_id;
	rcpt_id_T rcpt_ids[KEYS_MAX];
	edbc_ctx_P edbc_ctx;
	edbc_tnode_T edbc_tnode;
	edbc_node_T edbc_node_search;

	sm_memzero(&edbc_node_search, sizeof(edbc_node_search));
	sm_memzero(&edbc_tnode, sizeof(edbc_tnode));
	ECNL_INIT(&(edbc_tnode.ectn_hd));
	ECNL_APP(&(edbc_tnode.ectn_hd), &edbc_node_search);

	ret = edbc_open(&edbc_ctx, 0, 0);
	SM_TEST_E(ret == SM_SUCCESS);

	for (a = 0; a < KEYS_MAX; a++) {
		next_try[a] = a;
		sm_snprintf(ta_id, sizeof(ta_id), SMTPS_STID_FORMAT,
			(ulonglong_T) a, 0);
		sm_snprintf(rcpt_ids[a], sizeof(rcpt_ids[a]),
			SMTP_RCPTID_FORMAT, ta_id, 0);
	}

	for (u = 0, a = 0; u < nodes; u++) {
		sm_snprintf(rcpt_ids[a], sizeof(rcpt_ids[a]),
			SMTP_RCPTID_FORMAT, "rcpt-id-filler-text", u);
		ret = edbc_add(edbc_ctx, rcpt_ids[a], next_try[a], check);
		if (maxn == 0 || u < maxn) {
			if (sm_is_err(ret))
				fprintf(stderr, "u=%u, a=%d, ret=%#x\n", u, a, ret);
			SM_TEST_E(!sm_is_err(ret));
		}
		else
			SM_TEST_E(sm_is_err(ret));
		++a;
		if (a >= KEYS_MAX)
			a = 0;
	}

  error:
	edbc_close(edbc_ctx);
	return;
}

int
main(int argc, char *argv[])
{
	int c;
	uint maxn, nodes;
	bool check;

	maxn = 0;
	nodes = 10000;
	check = true;

	while ((c = getopt(argc, argv, "ch:m:s:")) != -1) {
		switch (c) {
		  case 'c':
			check = false;
			break;
		  case 'h':
			c = strlen(optarg);
			if (c >= SMTP_RCPTID_LEN - 1) {
				uint h;

				h = edbc_hash(optarg);
				fprintf(stderr, "h=%u\n", h);
				return 0;
			}
			else
				fprintf(stderr, "len(%s)=%d\n", optarg, c);
			return -1;
			break;
		  case 'm':
			maxn = strtoul(optarg, NULL, 0);
			break;
		  case 's':
			nodes = strtoul(optarg, NULL, 0);
			break;
		  default:
			return EX_USAGE;
		}
	}

	sm_test_begin(argc, argv, "test edbc perf 0");
	test(maxn, nodes, check);
	return sm_test_end();
}
