/*
 * Copyright (c) 2010 Claus Assmann
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the license/LICENSE.3C file which can be found at the
 * top level of this source code distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: isenhsc.c,v 1.3 2011/03/03 16:34:00 ca Exp $")
#include "sm/ctype.h"
#include "sm/enhsc.h"

/*
**  ISENHSC -- check whether a string starts with an enhanced status code
**
**	Parameters:
**		s -- string with possible enhanced status code.
**		delim -- delim for enhanced status code.
**
**	Returns:
**		0  -- no enhanced status code.
**		>4 -- length of enhanced status code.
*/

int
isenhsc(const char *s, int delim)
{
	uint h, l;

	if (NULL == s)
		return 0;
	if (s[0] != '2' && s[0] != '4' && s[0] != '5')
		return 0;
	if (s[1] != '.')
		return 0;
	l = 2;
	for (h = 0; h < 3 && ISDIGIT(s[l + h]); h++)
		;
	if (h == 0 || s[l + h] != '.')
		return 0;
	l += h + 1;
	for (h = 0; h < 3 && ISDIGIT(s[l + h]); h++)
		;
	if (h == 0 || s[l + h] != delim)
		return 0;

	return l + h;
}
