/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: rcbgetipaddr.c,v 1.2 2011/01/30 21:49:06 ca Exp $")
#include "sm/limits.h"
#include "sm/assert.h"
#include "sm/magic.h"
#include "sm/memops.h"
#include "sm/rpool.h"
#include "sm/str-int.h"
#include "sm/rcb.h"
#include "sm/net.h"

/*
**  SM_RCB_GETIPADDR -- Read data from rcb into ipaddr
**
**	Parameters:
**		rcb -- sm_rcb_P object to read from.
**		ipaddr -- ipaddr to fill.
**
**	Returns:
**		usual sm_error code; SM_E_OVFLW_NS, EPROTONOSUPPORT
*/

sm_ret_T
sm_rcb_getipaddr(sm_rcb_P rcb, sm_ipaddr_P ipaddr)
{
	sm_ret_T ret;
	int len;
	uint32_t family;
	uchar *buf;

	SM_IS_RCB(rcb);
#if SM_RCB_CHECK
	SM_REQUIRE(rcb->sm_rcb_state == SM_RCB_DEC);
#endif
#if PARANOID
	SM_REQUIRE(rcb->sm_rcb_base != NULL);
#endif
	SM_REQUIRE(ipaddr != NULL);

	ret = sm_rcb_getuint32(rcb, &family);
	if (sm_is_err(ret))
		goto error;

	ipaddr->ipa_family = family;
	switch (family) {
	  case AF_INET:
		len = sizeof(ipaddr->ipa_addr.ipv4_addr);
		buf = (uchar *)&ipaddr->ipa_addr.ipv4_addr;
		break;

#if MTA_NETINET6
	  case AF_INET6:
		len = sizeof(ipaddr->ipa_addr.ipv6_addr);
		buf = (uchar *)&ipaddr->ipa_addr.ipv6_addr;
		break;
#endif
	  default:
		ret = sm_error_perm(SM_EM_NONE, EPROTONOSUPPORT);
		goto error;
	}
	ret = sm_rcb_getn(rcb, buf, len);
	if (sm_is_err(ret))
		goto error;

	return SM_SUCCESS;

  error:
	return ret;
}
