/*
 *  producer program trying to emulate smtps (not particularly well...)
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-producer-0.c,v 1.11 2010/09/08 02:18:55 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/memops.h"
#include "sm/heap.h"
/* #include "sm/test.h" */
#include "sm/socket.h"
#include "sm/io.h"
#include "statethreads/st.h"
#include "sm/stsock.h"

#include "t-prdcns-0.h"

#define sm_sock_close(fd)	un_st_socket_close(fd)
#define SM_INVALID_SOCKET	NULL
#define SM_IS_INVALID_SOCKET(fd)	((fd) == NULL)

/* create dynamically? */
static int tids[T_QS];
#define FREE_ENTRY	(-9999)

static int max_threads = T_QS;
static int cur_threads = T_QS;
static int waiting = 0;
static long nsnd = 0;

static st_netfd_t fd;

static int
ss_connect(void)
{
	int i;
	sm_ret_T ret;

	if (Verbose > 1)
		dbg_print("clt: connect\n");
	fd = SM_INVALID_SOCKET;
	ret = sm_err_perm(EINVAL);

	/* connect to server */
	for (i = 1; i <= 5; i++) {
		ret = un_st_client_connect(MYSOCK, -1, &fd);
		if (sm_is_err(ret)) {
			dbg_print("clt: connect=%m\n", ret);
			if (sm_error_value(ret) == ECONNREFUSED) {
				st_sleep(i);
				continue;
			}
			break;
		}
		else
			break;
	}
	return ret;
}

static int
t_init(uint sz)
{
	int r;

	r = st_init();
	SM_TEST(r >= 0);
	if (r < 0)
		exit(r);
	ss_connect();
	return 0;
}

static int
p_sndv(int32_t v)
{
	int p;
	char buf[PBUFSZ];

	*((int *)buf) = v;
    p = st_write(fd, buf, PBUFSZ, 1000);
	if (p <= 0)
		return sm_err_perm(errno);
	++q_sent;
	return p;
}

static int
p_rcvv(int32_t *pv)
{
	int r;
	char buf[PBUFSZ];

	r = st_read(fd, buf, sizeof(buf), -1);
	if (r <= 0)
		return sm_err_perm(errno);
	++q_rcvd;
	*pv = *((int *)buf);
	return r;
}

static void *
p_snd(void *arg)
{
	while (nsnd < reps) {
		int i, j, r;
		bool lw;

		if (stop) return (void *)0;
		if (Verbose > 1)
				dbg_print(
					"p_snd: window=%d, waiting=%d, nsnd=%ld, reps=%ld\n",
					cur_threads, waiting, nsnd, reps);
		lw = false;
		while (waiting >= max_threads || cur_threads < 1) {
#define USLEEPT 100
			st_usleep(100);
			if (stop)
				return (void *)0;
			if (!lw) {
				dbg_print("p_snd: usleep=%d, window=%d, waiting=%d\n",
					USLEEPT, waiting, cur_threads);
				lw = true;
			}
		}

		for (i = 0, j = 0; i < max_threads && j < cur_threads; i++) {
			if (Verbose > 2)
				dbg_print("p_snd: i=%d, tids[%d]=%d\n",
					i, i, tids[i]);
			if (tids[i] == FREE_ENTRY) {
				tids[i] = i;
				r = p_sndv(tids[i]);
				if (r < 0) {
					dbg_print("p_snd: p_sndv=%m\n", r);
					break;
				}
				++nsnd;
				++j;
				++waiting;
				if (Verbose > 1)
					dbg_print(
						"p_snd: i=%d, window=%d, j=%d, waiting=%d\n",
						i, cur_threads, j, waiting);
			}
		}
	}

	return (void *)0;
}

static void *
p_rcv(void *arg)
{
	int r;
	int32_t v;

	while (true) {
		if (stop)
			return (void *)0;
		if (Verbose > 1)
			dbg_print("p_rcv: waiting=%d\n", waiting);
		r = p_rcvv(&v);
		if (r < 0) {
			dbg_print("p_rcv: p_rcvv=%m, stop=1\n", r);
			stop = 1;
		}
		if (v >= 0) {
			if (tids[v] != FREE_ENTRY) {
				--waiting;
				tids[v] = FREE_ENTRY;
			}
			if (Verbose > 1)
				dbg_print("p_rcv: v=%d\n", v);
		}
		else {
			/*
			 *  set window size to -v?
			 *  no, it should be:
			 *  returned window size - not acknowledged data
RFC 813:
It is the offered window which is actually transmitted back from the
receiver to the sender. The sender uses the offered window to compute a
different value, the "usable window", which is the offered window minus
the amount of outstanding unacknowledged data.

			 */
			if (Verbose > 1)
				dbg_print(
					"p_rcv: offered_window=%d, old_window=%d, waiting=%d, new_window=%d\n",
					-v, cur_threads, waiting, -v - waiting);
			cur_threads = -v - waiting;
		}
	}

	return (void *)0;
}

static void
testev(uint sz)
{
	int ret, i;

	ret = t_init(T_QS);
	if (ret != 0)
		return;
	if (SM_IS_INVALID_SOCKET(fd))
		return;
	for (i = 0; i < max_threads; i++)
		tids[i] = FREE_ENTRY;

	ret = T_QS;
	if (st_thread_create(p_rcv, (void *)ret, 0, 0) == NULL) {
		dbg_print("testev: ERROR: create=%d\n", errno);
		return;
	}
	p_snd((void *)ret);
	while (waiting * 100 > 25 * sz) {
		st_usleep(1000);
		dbg_print("testev: waiting=%d, window=%d\n", waiting, cur_threads);
	}
	dbg_print("testev: nsnd=%ld, waiting=%d\n", nsnd, waiting);
}

static void
usage(const char *prg)
{
	sm_io_fprintf(smioerr, "%s: simple producer test\n", prg);
	sm_io_fprintf(smioerr, "usage %s [options]\n", prg);
	sm_io_fprintf(smioerr, "options\n");
	sm_io_fprintf(smioerr, "-r n  send n items in total\n");
	sm_io_fprintf(smioerr, "-s n  use n for size of window\n");
	sm_io_fprintf(smioerr, "-V    increase verbosity\n");
}

int
main(int argc, char *argv[])
{
	int c;
	uint sz;

	sz = 100;
	reps = 2 * sz;
	while ((c = getopt(argc, argv, "r:s:V")) != -1)
	{
		switch (c)
		{
		  case 'r':
			reps = atoi(optarg);
			break;
		  case 's':
			sz = atoi(optarg);
			max_threads = cur_threads = sz;
			break;
		  case 'V':
			Verbose++;
			break;
		  default:
			usage(argv[0]);
			return(1);
		}
	}
	testev(sz);
	return 0;
}
