/*
 * Copyright (c) 2010 Claus Assmann
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the license/LICENSE.3C file which can be found at the
 * top level of this source code distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: qm_avbl.c,v 1.2 2011/01/30 21:48:50 ca Exp $")
#include "sm/common.h"
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/qmgr.h"
#include "sm/qmgr-int.h"
#include "qmgr.h"
#include "sm/pthread.h"

/*
**  QM_AVBL_SLOTS -- how many "slots" are available for SMTPS?
**
**	Parameters:
**		qmgr_ctx -- QMGR context
**
**	Returns:
**		>=0: number of available entries
**		<0: usual sm_error code
*/

sm_ret_T
qm_avbl_slots(qmgr_ctx_P qmgr_ctx)
{
	sm_ret_T r1, r2;

	SM_IS_QMGR_CTX(qmgr_ctx);
	r1 = aq_get_avail(qmgr_ctx->qmgr_aq, THR_NO_LOCK);
	if (sm_is_err(r1))
		return r1;
	r2 = iqdb_avail(qmgr_ctx->qmgr_iqdb);
	if (sm_is_err(r2))
		return r2;
	return SM_MIN(r1, r2);
}
