#!/bin/sh
# $Id: t-prot-2.sh,v 1.3 2013/11/30 02:27:24 ca Exp $
#
# ------------------------------------------------------------
# Description:
# Script to test "protected recipients": IPv6
# Uses t-acc-0, aliases and access map, t-mm
# ------------------------------------------------------------
#

# path to programs
P=..
TPRG=./t-acc-0
SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../smar
fi
. ${SD}/common.sh

chkctoprg ${TPRG} MTA_NETINET6

RES=${BASEOUT}.addr

if test X"`uname`" != "XOSF1"
then
while getopts d:s:V FLAG
do
  case "${FLAG}" in
    d) DBG="-d ${OPTARG}";;
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# create map for smar
rm -f ${ALIMAP}
cat > ${ALI} <<EOF
list3:	<user1-3@l3-1.dom> <user2-3@l3-2.dom>
EOF
${MM} < ${ALI}
if test $? -ne 0
then
  echo "$0: t-mm failed"
  exit 1
fi

rm -f ${ACCMAP}
# create access map for tests
cat > ${ACC} <<EOF
protectedrcpt:list3	from:<moderator3@local.dom> cltaddr:fec0:0:0:0:2fd:87fe:1e72:1d4f cltaddr:fed0:
EOF
${VALGRIND} ${MM} -t'	' -F ${ACCMAP} < ${ACC}

# create mailertable (save old version?)
${CAT} > ${MT} <<EOF
local.dom	lmtp:
b.c	lmtp:
EOF

TEST=1
# ----------------------------------------
# client allowed
${VALGRIND} ${TPRG} ${DBG} -p '<>' -i fec0:0:0:0:2fd:87fe:1e72:1d4f '<list3@local.dom>' > ${OUT} || ERRS=`expr ${ERRS} + 1 `
cat > ${OK} <<EOF
ara_status=0
EOF
if diff ${OK} ${OUT}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

TEST=2
# ----------------------------------------
# client not allowed
${VALGRIND} ${TPRG} ${DBG} -p '<>' -i fec0:0:0:0:2fd:87fe:1e72:1d4e '<list3@local.dom>' > ${OUT} || ERRS=`expr ${ERRS} + 1 `
cat > ${OK} <<EOF
ara_status=550
EOF
if diff ${OK} ${OUT}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

TEST=3
# ----------------------------------------
# client allowed
${VALGRIND} ${TPRG} ${DBG} -p '<>' -i fed0:0:0:0:2fd:87fe:1e72:1d4f -mD '<list3@local.dom>' > ${OUT} || ERRS=`expr ${ERRS} + 1 `
cat > ${OK} <<EOF
ara_status=0
EOF
if diff ${OK} ${OUT}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

TEST=4
# ----------------------------------------
# client not allowed
${VALGRIND} ${TPRG} ${DBG} -p '<>' -i fec1:0:0:0:2fd:87fe:1e72:1d4f -mD '<list3@local.dom>' > ${OUT} || ERRS=`expr ${ERRS} + 1 `
cat > ${OK} <<EOF
ara_status=550
EOF
if diff ${OK} ${OUT}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

rm -f ${OUT} ${RES} ${OK}

exit ${ERRS}
